  MEMBER
  
  include('audit.inc')
  include('equates.clw')
  
  MAP
    OpenLogfile(BYTE Create=true), BYTE, PROC
    CloseLogfile()
    LOG:AssignFilename(STRING Filename)
  END  

OpenLogCounter  LONG(0)

LogfilePath   STRING(MAX_PATH)
Logfile       FILE,DRIVER('TopSpeed'),NAME(LogfilePath),PRE(LOG),CREATE
KeyFilename     KEY(LOG:Filename),NOCASE
Record          RECORD
Filename          STRING(40)
Records           LONG
                END
              END

!-------------------------------------------------------------------
LOG:AssignFilename  PROCEDURE(STRING Filename)
!-------------------------------------------------------------------
  CODE
    IF len(clip(Filename))<=40 THEN
      LOG:Filename = Filename
    ELSE  
      LOG:Filename = sub(Filename,len(clip(Filename))-39,40)
    END  

!-------------------------------------------------------------------
ICON_Logfile    PROCEDURE(<STRING Path>)
!-------------------------------------------------------------------
  CODE
    IF ~omitted(1) THEN
      IF LogfilePath THEN
        assert(0,'TPS.repair, PROCEDURE Logfile(): Logfile Pathname already set.')
      ELSE  
        LogfilePath = Path
      END  
    END
    RETURN LogfilePath

!-------------------------------------------------------------------
ICON_FileRecords    PROCEDURE(STRING Filename)
!-------------------------------------------------------------------
Records LONG(-1)

  CODE
    IF ~LogfilePath THEN
      assert(0,'TPS.repair, PROCEDURE FileRecords(): Logfile Pathname unknown.')
    ELSE  
      IF OpenLogfile(false) THEN 
        LOG:AssignFilename(Filename)
        get(Logfile,LOG:KeyFilename)
        IF ~errorcode() THEN
          Records=LOG:Records
        END  
        CloseLogfile()
      END
    END
    RETURN Records

!-------------------------------------------------------------------
OpenLogfile    PROCEDURE(BYTE create=true)
!-------------------------------------------------------------------
  CODE
    IF ~status(Logfile) THEN
?     assert(OpenLogCounter=0,'TPS.repair: PROCEDURE OpenLogfile() Logfile has been closed externally')
      OpenLogCounter=0
      IF ~exists(LogfilePath) AND create THEN
        create(Logfile)
      END
      open(Logfile,42H) 
?     assert(errorcode()=0,'TPS.repair: PROCEDURE OpenLogfile() Logfile could not be created/opened')
    END
    OpenLogCounter+=1
    RETURN choose(status(Logfile)<>0)
  
!-------------------------------------------------------------------
CloseLogfile    PROCEDURE
!-------------------------------------------------------------------
  CODE
    OpenLogCounter-=1
?   assert(OpenLogCounter>=0,'TPS.repair: PROCEDURE CloseLogfile() mismatched openLogfile/closeLogfile')
    IF status(Logfile) AND OpenLogCounter<=0 THEN
      close(Logfile) 
?     assert(errorcode()=0,'TPS.repair: PROCEDURE CloseLogfile() Logfile could not be closed')
    END
  
!-------------------------------------------------------------------
CICON_Audit.CONSTRUCT       PROCEDURE()
!-------------------------------------------------------------------
  CODE
    SELF.Watchfile &= NULL
    clear(SELF.LogfilePosition)

!-------------------------------------------------------------------
CICON_Audit.DESTRUCT       PROCEDURE()
!-------------------------------------------------------------------
  CODE
    IF ~(SELF.Watchfile &= NULL) THEN
      closeLogfile()
      callback(SELF.Watchfile,SELF.FileCallBackInterface,false)
    END  
    
!-------------------------------------------------------------------
CICON_Audit.init       PROCEDURE(FILE F,STRING Filename)
!-------------------------------------------------------------------
  CODE
    IF ~LogfilePath THEN
?     assert(0,'TPS.repair, PROCEDURE CAudit.init(): Logfile Pathname not set')
    ELSE  
      SELF.Watchfile &= F
      IF openLogfile() THEN
        clear(LOG:Record)
        LOG:AssignFilename(Filename)
        get(Logfile,LOG:KeyFilename)
        IF errorcode()=35 THEN
          IF status(SELF.Watchfile) THEN
            LOG:Records=records(SELF.Watchfile)
          END  
          add(Logfile)
        END  
?       assert(errorcode()=0,'TPS.repair: PROCEDURE CAudit.init() Logfile entry could not be added for file ' & Filename)
        IF ~errorcode() THEN
          SELF.LogfilePosition=position(Logfile)
          callback(SELF.Watchfile,SELF.FileCallBackInterface)
        END  
      END
    END    

!-------------------------------------------------------------------
CICON_Audit.suspend         PROCEDURE()
!-------------------------------------------------------------------
  CODE
    SELF.Suspended=true

!-------------------------------------------------------------------
CICON_Audit.resume          PROCEDURE()
!-------------------------------------------------------------------
  CODE
    SELF.Suspended=true
    SELF.update()

!-------------------------------------------------------------------
CICON_Audit.update          PROCEDURE()
!-------------------------------------------------------------------
  CODE
    reget(Logfile,SELF.LogfilePosition)
    IF errorcode()=0 THEN
      LOG:Records=records(SELF.Watchfile)
      put(Logfile)
    END
?   assert(errorcode()=0,'TPS.repair: PROCEDURE CAudit.update() Logfile entry could not be updated for file ' & SELF.Watchfile{prop:name})

!-------------------------------------------------------------------
CICON_Audit.FileCallBackInterface.FunctionCalled PROCEDURE(SIGNED opCode, *Params Parameters, *CSTRING ErrCode, *CSTRING ErrMsg)
!-------------------------------------------------------------------
  CODE
    RETURN true

!-------------------------------------------------------------------
CICON_Audit.FileCallBackInterface.FunctionDone   PROCEDURE(SIGNED opCode, *Params Parameters, *CSTRING ErrCode, *CSTRING ErrMsg)
!-------------------------------------------------------------------
  CODE
    IF ~SELF.suspended THEN
      CASE opCode
      OF   DriverOp:OPEN
      OROF DriverOp:ADD
      OROF DriverOp:APPEND
      OROF DriverOp:DELETE
        IF ~error() THEN        ! else we would overwrite the "errorcode" with the "put" in update
          SELF.update()
        END  
      END
    END
    RETURN TRUE